/**
 * @file DecAgentG.cpp
 * @author ...
 * ...
 * @@tag Decision@vle.extension.decision @@namespace:tp5_1_correction;class:DecAgentG;f:DvtStade|Rain|;p:periodeSeche|eauDisponible|finPeriodeSemis|maturitePlus30jours|solPortant|recolteNonEffectuee|maturiteAtteinte|;of:semis|recolte|irrigation|;pf:DecAgentG;ak:ackIrrigation|ackRecolte|;pm:apportEau,double,30|quantiteEauPeriodeSeche,double,10|quantiteEauSolPortant,double,10|stockEau,double,50|@@end tag@@
  */

#include <vle/extension/decision/Agent.hpp>
#include <vle/extension/decision/Activity.hpp>
#include <vle/extension/decision/KnowledgeBase.hpp>
#include <vle/utils/Package.hpp>
#include <sstream>
#include <numeric>
#include <fstream>
//@@begin:include@@
enum CROP_STATE { SOWING /*0*/, EMERGENCE /*1*/, MAX_LAI /*2*/, FLOWERING /*3*/,
	     CRITICAL_GRAIN_ABORTION /*4*/, LEAF_SENESCENCE /*5*/, MATURITY /*6*/,  BARE_SOIL /*7*/};//@@end:include@@
namespace vd = vle::devs;
namespace vv = vle::value;
namespace ved = vle::extension::decision;
namespace vu = vle::utils;
namespace vmd = vle::extension::decision;
namespace tp5_1_correction {

class DecAgentG : public vmd::Agent
{
public:
    DecAgentG(
        const vd::DynamicsInit& init,
        const vd::InitEventList& evts)
        : vmd::Agent(init, evts)

,
//@@begin:factsInitDvtStade@@
mStade(BARE_SOIL),
mDateDebutMaturite(0.0),
mRecolteEffectuee(false)

















//@@end:factsInitDvtStade@@

,
//@@begin:factsInitRain@@
mPluies(5, 0.)























//@@end:factsInitRain@@

    {
//@@begin:customconstructor@@

//@@end:customconstructor@@
        //Parameters managed by the plugin//@@begin:constructor@@
if (evts.exist("apportEau"))
apportEau = evts.getDouble("apportEau");
else
throw vle::utils::ModellingError("ParameterapportEaunot found");

if (evts.exist("quantiteEauPeriodeSeche"))
quantiteEauPeriodeSeche = evts.getDouble("quantiteEauPeriodeSeche");
else
throw vle::utils::ModellingError("ParameterquantiteEauPeriodeSechenot found");

if (evts.exist("quantiteEauSolPortant"))
quantiteEauSolPortant = evts.getDouble("quantiteEauSolPortant");
else
throw vle::utils::ModellingError("ParameterquantiteEauSolPortantnot found");

if (evts.exist("stockEau"))
stockEau = evts.getDouble("stockEau");
else
throw vle::utils::ModellingError("ParameterstockEaunot found");

//@@end:constructor@@
        //KnowledgeBase Inititialisation
        addFacts(this) +=
            F("DvtStade", &DecAgentG::DvtStade),
            F("Rain", &DecAgentG::Rain);

        addPredicates(this) +=
            P("periodeSeche", &DecAgentG::periodeSeche),
            P("eauDisponible", &DecAgentG::eauDisponible),
            P("finPeriodeSemis", &DecAgentG::finPeriodeSemis),
            P("maturitePlus30jours", &DecAgentG::maturitePlus30jours),
            P("solPortant", &DecAgentG::solPortant),
            P("recolteNonEffectuee", &DecAgentG::recolteNonEffectuee),
            P("maturiteAtteinte", &DecAgentG::maturiteAtteinte);
        addOutputFunctions(this) +=
            O("semis", &DecAgentG::semis),
            O("recolte", &DecAgentG::recolte),
            O("irrigation", &DecAgentG::irrigation);

        addAcknowledgeFunctions(this) +=
            A("ackIrrigation", &DecAgentG::ackIrrigation),
            A("ackRecolte", &DecAgentG::ackRecolte);

        //Plan initialisation
        std::string dataPackageParam;
        if (evts.exist("PackageName"))
          dataPackageParam = evts.getString("PackageName");
        else
          throw vle::utils::ModellingError(
             "Package where to find the data is not set");
        vle::utils::Package mPack(dataPackageParam);
        std::string filePath =
        mPack.getDataFile("DecAgentG.txt");
        std::ifstream fileStream(filePath.c_str());
        KnowledgeBase::plan().fill(fileStream);
    }

    virtual ~DecAgentG()
    {}

//@@begin:definition@@
double apportEau;
double quantiteEauPeriodeSeche;
double quantiteEauSolPortant;
double stockEau;
//@@end:definition@@
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Predicates
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
bool periodeSeche() const
{//@@begin:predicatesFunctionperiodeSeche@@
return std::accumulate(mPluies.begin(), mPluies.begin() + 5, 0.)
            <= quantiteEauPeriodeSeche;//@@end:predicatesFunctionperiodeSeche@@
}

bool eauDisponible() const
{//@@begin:predicatesFunctioneauDisponible@@
return stockEau > 0;//@@end:predicatesFunctioneauDisponible@@
}

bool finPeriodeSemis() const
{//@@begin:predicatesFunctionfinPeriodeSemis@@
return activite("Semis").finish() == currentTime();//@@end:predicatesFunctionfinPeriodeSemis@@
}

bool maturitePlus30jours() const
{//@@begin:predicatesFunctionmaturitePlus30jours@@
return currentTime() == mDateDebutMaturite + 30;//@@end:predicatesFunctionmaturitePlus30jours@@
}

bool solPortant() const
{//@@begin:predicatesFunctionsolPortant@@
return std::accumulate(mPluies.begin(), mPluies.begin() + 3, 0.)
        <= quantiteEauSolPortant;//@@end:predicatesFunctionsolPortant@@
}

bool recolteNonEffectuee() const
{//@@begin:predicatesFunctionrecolteNonEffectuee@@
 return not mRecolteEffectuee;//@@end:predicatesFunctionrecolteNonEffectuee@@
}

bool maturiteAtteinte() const
{//@@begin:predicatesFunctionmaturiteAtteinte@@
 return (mDateDebutMaturite > 0);//@@end:predicatesFunctionmaturiteAtteinte@@
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Facts
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
void DvtStade(const vv::Value& value)
{//@@begin:factsFunctionDvtStade@@
 int newstade = std::floor(value.toDouble().value());

 if (newstade == MATURITY && mStade != MATURITY) {
	mDateDebutMaturite = mCurrentTime;
}

mStade = (CROP_STATE) newstade;//@@end:factsFunctionDvtStade@@
}

void Rain(const vv::Value& value)
{//@@begin:factsFunctionRain@@
std::rotate(mPluies.rbegin(), mPluies.rbegin() + 1, mPluies.rend());
mPluies[0] = value.toDouble().value();//@@end:factsFunctionRain@@
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Output Functions
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
void semis(
const std::string& name,
const ved::Activity& activity,
vd::ExternalEventList& output) {//@@begin:outputFunctionsemis@@
if (activity.isInStartedState()) {
        vd::ExternalEvent* evt = new vd::ExternalEvent("semis");
        evt->putAttribute("name", new vv::String(name));
        evt->putAttribute("activity", new vv::String(name));
        evt->putAttribute("value", new vv::Double(1.));
        output.push_back(evt);
}//@@end:outputFunctionsemis@@
}

void recolte(
const std::string& name,
const ved::Activity& activity,
vd::ExternalEventList& output) {//@@begin:outputFunctionrecolte@@
if (activity.isInStartedState()) {
        vd::ExternalEvent* evt = new vd::ExternalEvent("recolte");
        evt->putAttribute("name", new vv::String(name));
        evt->putAttribute("activity", new vv::String(name));
        evt->putAttribute("value", new vv::Double(1.));
        output.push_back(evt);
}//@@end:outputFunctionrecolte@@
}

void irrigation(
const std::string& name,
const ved::Activity& activity,
vd::ExternalEventList& output) {//@@begin:outputFunctionirrigation@@
if (activity.isInStartedState()) {
        vd::ExternalEvent* evt = new vd::ExternalEvent("irrigation");
        evt->putAttribute("name", new vv::String("Irrigation"));
        evt->putAttribute("activity", new vv::String(name));
        evt->putAttribute("value",
                                                 new vv::Double(std::min(apportEau, stockEau)));
        output.push_back(evt);
}//@@end:outputFunctionirrigation@@
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Acknowledge functions
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
void ackIrrigation(const std::string&/*activityname*/, const ved::Activity& /*activity*/) {//@@begin:ackFunctionackIrrigation@@
stockEau -= apportEau;//@@end:ackFunctionackIrrigation@@
}

void ackRecolte(const std::string&/*activityname*/, const ved::Activity& /*activity*/) {//@@begin:ackFunctionackRecolte@@
mRecolteEffectuee = true;//@@end:ackFunctionackRecolte@@
}

private:
        //Custom members//@@begin:custommembers@@
const ved::Activity& activite(const std::string& nom) const
{
        return activities().get(nom)->second;
}
//@@end:custommembers@@

//@@begin:factsAttributeDvtStade@@
CROP_STATE mStade; vd::Time mDateDebutMaturite; bool mRecolteEffectuee;//@@end:factsAttributeDvtStade@@

//@@begin:factsAttributeRain@@
std::vector<double> mPluies;//@@end:factsAttributeRain@@



/*@@begin:hierarchicalPreds@@
@@end:hierarchicalPreds@@*/
};

} // namespace tp5_1_correction

DECLARE_DYNAMICS(tp5_1_correction::DecAgentG)
