/**
 * @file vle/share/template/package/src/simple.cpp
 * @author The VLE Development Team
 */

/*
 * VLE Environment - the multimodeling and simulation environment
 * This file is a part of the VLE environment (http://vle.univ-littoral.fr)
 * Copyright (C) 2003 - 2009 The VLE Development Team
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#define BOOST_TEST_MAIN
#define BOOST_AUTO_TEST_MAIN
#define BOOST_TEST_DYN_LINK
#define BOOST_TEST_MODULE test_tp
#include <boost/test/unit_test.hpp>
#include <boost/test/auto_unit_test.hpp>
#include <boost/test/floating_point_comparison.hpp>
#include <ostream>

#include <vle/vle.hpp>
#include <vle/value/Map.hpp>
#include <vle/value/Matrix.hpp>
#include <vle/vpz/Vpz.hpp>
#include <vle/utils/Path.hpp>
#include <vle/utils/Package.hpp>
#include <vle/manager/Manager.hpp>
#include <vle/manager/Simulation.hpp>

namespace vz = vle::vpz;
namespace vu = vle::utils;
namespace vm = vle::manager;
namespace va = vle::value;

BOOST_AUTO_TEST_CASE(test_tp)
{
    vu::Package pack("tp5_1_correction");
    vz::Vpz vpz(pack.getExpFile("2CV-decision.vpz"));
    vz::Outputs::iterator itb =
            vpz.project().experiment().views().outputs().begin();
    vz::Outputs::iterator ite =
            vpz.project().experiment().views().outputs().end();
    for(;itb!=ite;itb++){
        itb->second.setLocalStream("", "storage", "vle.output");
    }

    //simulation
    vu::ModuleManager man;
    vm::Error error;
    vm::Simulation sim(vm::LOG_NONE, vm::SIMULATION_NONE, NULL);
    va::Map *out = sim.run(new vz::Vpz(vpz), man, &error);

    //checks that simulation has succeeded
    BOOST_REQUIRE_EQUAL(error.code, 0);
    //checks the number of views
    BOOST_REQUIRE_EQUAL(out->size(),1);
    //checks the selected view
    const va::Matrix& view = out->getMatrix("vueBilan");
    BOOST_REQUIRE_EQUAL(view.columns(),7);
    BOOST_REQUIRE_EQUAL(view.column(0).size(),1);

    //check Date recolte
    BOOST_REQUIRE_EQUAL(view.getString(1,0), "1961-09-08");
}
